
// (c) COPYRIGHT URI/MIT 1996,1997,1999
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

// Implementation of the Gui class. A GUI is a program runing in a subprocess
// which reads commands from its stdin and writes responses to stdout. The
// GUI presents inforamtion to the user using the usual combination of
// graphical candy.
//
// jhrg 6/21/96

// $Log: Gui.cc,v $
// Revision 1.15  1999/08/09 18:27:34  jimg
// Merged changes from Brent for the Gui code (progress indicator)
//
// Revision 1.14.8.2  1999/08/03 04:29:10  brent
// Gui.cc now contains Tcl code, an external Tcl file is not needed
//
// Revision 1.14.8.1  1999/07/29 05:46:17  brent
// call Tcl / GUI directly from Gui.cc, abandon expect, and consolidate Tcl files
//
// Revision 1.14  1999/04/29 02:29:30  jimg
// Merge of no-gnu branch
//
// Revision 1.13.14.2  1999/02/05 09:32:34  jimg
// Fixed __unused__ so that it not longer clashes with Red Hat 5.2 inlined
// math code.
//
// Revision 1.13.14.1  1999/02/02 21:56:59  jimg
// String to string version
//
// Revision 1.13  1997/06/09 20:58:47  jimg
// Added `etc/' to default GUI initialization command location
// (See _start_gui()).
//
// Revision 1.12  1997/04/18 17:26:22  jimg
// Changed include of String.h so that it uses `<>' quotes instead of `""'
// quotes since the latter messed up the dependencies in the Makefile.
//
// Revision 1.11  1997/03/05 08:26:51  jimg
// Fixed bug with _show_gui -vs- _user_wants_gui fields.
//
// Revision 1.10  1997/02/19 04:51:58  jimg
// Fixed bugs with _show_gui and _user_wants_gui.
//
// Revision 1.9  1997/02/10 02:34:40  jimg
// Changed _start_gui() call to dods_root to match new return type.
//
// Revision 1.8  1996/11/25 03:41:28  jimg
// Fixed dods_root so that it now uses the function (rather that a static
// global which won't be initialized when this library is linked using a
// program other than g++).
//
// Revision 1.7  1996/11/22 00:06:23  jimg
// Increased Expect timeout from 2 to 10 seconds.
//
// Revision 1.6  1996/11/18 21:59:32  jimg
// Error messages from command are more verbose - to simplify debugging.
//
// Revision 1.5  1996/11/13 18:58:40  jimg
// Added missing break statements to switch()s.
//
// Revision 1.4  1996/08/13 18:27:46  jimg
// Added not_used in definition of char rcsid[].
//
// Revision 1.3  1996/07/08 23:19:16  jimg
// Added support for the HAVE_EXPECT compile-time switch. When defined as 1
// support for expect is built, when 0 it is not. Thus the dap++ library may be
// built so that expect is not required to link programs. However, the GUI will
// work only when expect/tcl are present and when client are linked with those
// libraries in addition to libdap++.a
// Fixed comments.
// Fixed some warnings (but left some...).
//
// Revision 1.2  1996/06/22 00:04:23  jimg
// Removed forced DEBUG define.
//
// Revision 1.1  1996/06/21 23:12:29  jimg
// Created based on code that was in Connect.cc.
//

#ifdef __GNUG__
#pragma "implemenation"
#endif

#include "config_dap.h"

static char rcsid[] not_used = {"$Id: Gui.cc,v 1.15 1999/08/09 18:27:34 jimg Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <assert.h>

#include <sys/wait.h>

#if HAVE_EXPECT
#include <expect.h>
#endif

#include <string.h>	// for strlen()
#include <string>

// includes for the Tcl / Tk code

#include <tcl.h>
#include <tk.h>

#include "util.h"
#include "debug.h"
#include "Gui.h"

// Need the Tcl interpreter to be Global to this file

static Tcl_Interp * Interp = NULL;


Gui::Gui(const string cmd) 
    : _show_gui(true), _progress_visible(false)
{

    Tk_Window mainWindow;
    const char* g = getenv("DODS_USE_GUI");
    char *tcl_code, *read_tcl_script();

    string dods_gui = (g ? g : "");
    downcase(dods_gui);
    if (dods_gui == "no")
	_user_wants_gui = false; // Users explicitly say "no"...
    else
	_user_wants_gui = true; // Otherwise assume "yes".

    if (!_user_wants_gui)
	return;

    // now set up the Tcl interpter

    Interp = Tcl_CreateInterp();
    if (Interp == NULL) {
	_show_gui = false;
	return;
    }

    mainWindow = Tk_CreateWindow(Interp, NULL, "DODS", "");
    if (mainWindow == NULL) {
	_show_gui = false;
	Tcl_DeleteInterp(Interp);
	Interp = NULL;
        return;
    }

    if (Tcl_Init(Interp) == TCL_ERROR) {
	_show_gui = false;
	Tcl_DeleteInterp(Interp);
	Interp = NULL;
        return;
    }
    if (Tk_Init(Interp) == TCL_ERROR) {
	_show_gui = false;
	Tcl_DeleteInterp(Interp);
	Interp = NULL;
        return;
    }
    // call the functin containing the Tcl code
    tcl_code = read_tcl_script();
    int rc = Tcl_Eval(Interp, tcl_code);  
    if (rc != TCL_OK) {
	_show_gui = false;
	Tcl_DeleteInterp(Interp);
	Interp = NULL;
        return;
    }
    
    Tcl_Eval(Interp, (char *)(cmd.c_str()));	// execute the initial command
    // process all the events 
    do {
        rc = (Tcl_DoOneEvent(TCL_ALL_EVENTS | TCL_DONT_WAIT));
    }
    while (rc != 0);

}

Gui::Gui() 
    : _show_gui(true), _progress_visible(false)
{
    Tk_Window mainWindow;
    const char* g = getenv("DODS_USE_GUI");
    char *tcl_code, *read_tcl_script();

    string dods_gui = (g ? g : "");
    downcase(dods_gui);
    if (dods_gui == "no")
	_user_wants_gui = false; // Users explicitly say "no"...
    else
	_user_wants_gui = true; // Otherwise assume "yes".

    if (!_user_wants_gui)
	return;

    // now set up the Tcl interpter
    Interp = Tcl_CreateInterp();
    if (Interp == NULL) {
	_show_gui = false;
	fprintf(stderr, "bad Tcl Interp 1");
	return;
    }

    mainWindow = Tk_CreateWindow(Interp, NULL, "DODS", "");
    if (mainWindow == NULL) {
	cerr << "Could not create progress indicator window: "
	     << Interp->result << endl;
	_show_gui = false;
	Tcl_DeleteInterp(Interp);
	Interp = NULL;
        return;
    }
    
#if 1
    if (Tcl_Init(Interp) == TCL_ERROR) {
	_show_gui = false;
	cerr << "Could not initialize Tcl: " << Interp->result << endl;
	Tcl_DeleteInterp(Interp);
	Interp = NULL;
        return;
    }
    if (Tk_Init(Interp) == TCL_ERROR) {
	_show_gui = false;
	cerr << "Could not initialize Tk: " << Interp->result << endl;
	Tcl_DeleteInterp(Interp);
	Interp = NULL;
        return;
    }
#endif
    // call the functin containing the Tcl code
    tcl_code = read_tcl_script();
    int rc = Tcl_Eval(Interp, tcl_code);  
    if (rc != TCL_OK) {
	_show_gui = false;
	Tcl_DeleteInterp(Interp);
	Interp = NULL;
        return;
    }
}

Gui::~Gui()
{
    if (!show_gui() || (Interp == NULL)) // Don't try to stop what isn't running
	return;
    
    _show_gui = false;
    Tcl_DeleteInterp(Interp);
    Interp = NULL;
    return;
   
}

bool
Gui::progress_visible(int state)
{
    if (state < 0)
	return _progress_visible;
    else
	return (_progress_visible = state);
}

bool
Gui::command(string command)
{
    int rc;

    if (!show_gui())		// Bail if we're not supposed to be here...
	return true;

    DBG(cerr << "Sending the GUI: " << command << endl);

    Tcl_Eval(Interp, (char *)(command.c_str()));
    // process all the events 
    do {
        rc = (Tcl_DoOneEvent(TCL_ALL_EVENTS | TCL_DONT_WAIT));
    }
    while (rc != 0);
    // if this was the password command, we need to get the user
    // and password variables now 

    return true;
}

bool
Gui::simple_error(string error_msg)
{
    int rc;
    char message[200];

    if (!show_gui())		// Bail if were not supposed to be here...
	return true;

    DBG(cerr << "Sending the GUI: " << command << endl);

    sprintf(message, "simple_error \"%s\"", error_msg.c_str());
    Tcl_Eval(Interp, message);
    // process all the events 
    do {
        rc = (Tcl_DoOneEvent(TCL_ALL_EVENTS | TCL_DONT_WAIT));
    }
    while (rc != 0);

    return true;
}
// this version of command is called only to get the password
bool
Gui::command(string command, string * result1, string * result2)
{
    int rc;
    char *pass_var, *user_var;	// char ptrs for Tcl / Tk

    if (!show_gui())		// Bail if were not supposed to be here...
	return true;

    DBG(cerr << "Sending the GUI: " << command << endl);

    Tcl_Eval(Interp, (char *)(command.c_str()));
    // process all the events 
    do {
        rc = (Tcl_DoOneEvent(TCL_ALL_EVENTS | TCL_DONT_WAIT));
    }
    while (rc != 0);
    // if this was the password command, we need to get the user
    // and password variables now 
    user_var = Tcl_GetVar(Interp, "progress_usr", TCL_GLOBAL_ONLY);
    pass_var = Tcl_GetVar(Interp, "progress_pwd", TCL_GLOBAL_ONLY);
    // usr_var and pass_var are pointers into Tcl memory space, which
    // is quite volitaile, so lets copy them out 
    result1->resize(strlen(user_var));
    result2->resize(strlen(pass_var));
    result1->assign(user_var, strlen(user_var));
    result2->assign(pass_var, strlen(pass_var));

    return true;
}

bool
Gui::show_gui(int state)
{
    if (state < 0)
	return _show_gui && _user_wants_gui;
    else
	return (_show_gui = (bool)state) && _user_wants_gui;
}

char * 
read_tcl_script()
{

static char *src_code = "wm withdraw .        ;# Hide wish's toplevel window\n\
set font -*-courier-medium-r-normal--*-100-*-*-*-*-*-*\n\
set err_geometry +200+200\n\
set pass_geometry +200+200\n\
set progress_geometry +30+0\n\
set progress_bar_pos -1\n\
set progress_bar_rect 0\n\
set sign +\n\
proc progress_create {{text \"\"}} {\n\
global progress_geometry\n\
global font\n\
if {![winfo exists .progress]} {\n\
toplevel .progress\n\
wm withdraw .progress           ;# Hide the new toplevel window\n\
wm geometry .progress $progress_geometry\n\
wm title .progress \"dods transmission\"\n\
frame .progress.f -width 60m -height 20m\n\
label .progress.f.icon -bitmap info\n\
label .progress.f.msg -font $font -text $text -borderwidth 2 \\\n\
-width 22 -anchor w\n\
canvas .progress.f.bar -relief sunken -width 56m -height 3m -borderwidth 2\n\
pack .progress.f -padx 6 -pady 6\n\
pack .progress.f.icon -side left -padx 20 -pady 20 -anchor n\n\
pack .progress.f.msg -side top -padx 6 -pady 6 -fill y -expand true\n\
pack .progress.f.bar -side top -padx 6 -pady 6 -anchor s\n\
}\n\
}\n\
proc text {{new_text \"\"}} {\n\
.progress.f.msg configure -text $new_text\n\
}\n\
proc bar {{percent -2}} {\n\
global progress_bar_pos\n\
global progress_bar_rect\n\
global sign\n\
if {$progress_bar_rect != 0} {\n\
.progress.f.bar delete $progress_bar_rect\n\
}\n\
if {$percent < -1} {\n\
return\n\
}\n\
set bar_height [.progress.f.bar cget -height]\n\
set bar_width [.progress.f.bar cget -width]\n\
if {$percent == -1} {\n\
set box_width [expr $bar_width / 5]\n\
set incr_size [expr $box_width / 2]\n\
if {$progress_bar_pos == -1} {\n\
set new_pos 0\n\
} else {\n\
set new_pos [expr $progress_bar_pos $sign $incr_size]\n\
}\n\
if {[expr $new_pos + $box_width] > $bar_width} {\n\
set sign -\n\
set new_pos [expr $bar_width - $box_width]\n\
} elseif {$new_pos < 0} {\n\
set sign +\n\
set new_pos $box_width\n\
}\n\
#set new_pos [expr $new_pos $sign 4]\n\
set bar_height [expr $bar_height + 4]\n\
set progress_bar_rect [.progress.f.bar create \\\n\
rectangle [expr $new_pos - 4] 0 \\\n\
[expr $new_pos + $box_width + 4] $bar_height \\\n\
-fill Blue ]\n\
set progress_bar_pos $new_pos\n\
} else {\n\
set progress_bar_pos [expr $bar_width * ($percent / 100.0)]\n\
set progress_bar_pos [expr $progress_bar_pos + 4]\n\
set bar_height [expr $bar_height + 4]\n\
set progress_bar_rect [.progress.f.bar create rectangle 0 0 \\\n\
$progress_bar_pos $bar_height -fill Blue -width 0]\n\
}\n\
}\n\
proc popup {{text \"\"}} {\n\
if {![winfo exists .progress]} {\n\
progress_create $text\n\
} elseif {[string length $text] != 0} {\n\
text $text\n\
}\n\
wm deiconify .progress\n\
}\n\
proc popdown {} {\n\
if {[winfo exists .progress]} {\n\
wm withdraw .progress\n\
}\n\
}\n\
proc password {} {\n\
global font\n\
global pass_geometry\n\
global pass_button\011\011;# Used in -command; must be global \n\
global progress_pwd\011\011# these have to be global to work\n\
global progress_usr\n\
if {![winfo exists .pass]} {\n\
toplevel .pass\n\
wm withdraw .pass\n\
wm geometry .pass $pass_geometry\n\
wm title .pass \"Password\"\n\
frame .pass.f -relief groove -borderwidth 1\n\
label .pass.f.icon -bitmap questhead\n\
frame .pass.f.lf\n\
label .pass.f.lf.user -text \"Username:\"\n\
label .pass.f.lf.pass -text \"Password:\"\n\
frame .pass.f.ef\n\
entry .pass.f.ef.user -font $font\n\
entry .pass.f.ef.pass -font $font -show \"*\"\n\
frame .pass.bf -relief groove -borderwidth 1\n\
frame .pass.bf.def -relief sunken -borderwidth 1\n\
button .pass.bf.def.ok -text \"Ok\" -command { set pass_button \"OK\" }\n\
button .pass.bf.cancel -text \"Cancel\" \\\n\
-command {set pass_button \"CANCEL\" }\n\
pack .pass.f -side top -ipadx 10 -expand true -fill both \n\
pack .pass.f.icon -side left -ipadx 20 -ipady 20 -anchor n\n\
pack .pass.f.lf -side left -anchor n -pady 10 \n\
pack .pass.f.lf.user -side top -anchor e -padx 2 -pady 2 -fill x \\\n\
-expand true\n\
pack .pass.f.lf.pass -side top -anchor e -padx 2 -pady 2 -fill x \\\n\
-expand true\n\
pack .pass.f.ef -side left -anchor n -pady 10\n\
pack .pass.f.ef.user -side top -anchor w -padx 2 -pady 2 -fill x \\\n\
-expand true\n\
pack .pass.f.ef.pass -side top -anchor w -padx 2 -pady 2 -fill x \\\n\
-expand true\n\
pack .pass.bf -side top -fill both -expand true -anchor s\n\
pack .pass.bf.def -side left -padx 5 -pady 5 -expand true\n\
pack .pass.bf.def.ok -side top -padx 5 -pady 5\n\
pack .pass.bf.cancel -side left -padx 5 -pady 5 -expand true\n\
bind .pass <Return> { set pass_button \"OK\" }\n\
} else {\n\
.pass.f.ef.user delete 0 end\n\
.pass.f.ef.pass delete 0 end\n\
}\n\
focus .pass.f.ef.user \011;# Move to the first window\n\
wm deiconify .pass\n\
tkwait visibility .pass\n\
grab set .pass\n\
tkwait variable pass_button\n\
wm withdraw .pass\n\
grab release .pass\n\
puts $pass_button\n\
if {$pass_button == \"OK\"} {\n\
set progress_usr [.pass.f.ef.user get]\n\
set progress_pwd [.pass.f.ef.pass get]\n\
} elseif {$pass_button == \"Cancel\"} {\n\
set progress_usr \"\"         ;# set the values to a null char\n\
set progress_pwd \"\"\n\
}\n\
}\n\
proc simple_error {msg} {\n\
tk_dialog .tmp \"Error\" $msg error 0 \"Ok\"\n\
}\n";

	return (src_code);
}
